from typing import List
from uuid import UUID
from sqlalchemy.orm import Session
from models import PlaylistSong
from schemas import PlaylistSongSchema, PlaylistSongCreate

def get_playlist_songs(db: Session, playlist_id: UUID) -> List[PlaylistSongSchema]:
    """
    Retrieves all songs associated with a given playlist.

    Args:
        db (Session): The database session.
        playlist_id (UUID): The ID of the playlist.

    Returns:
        List[PlaylistSongSchema]: A list of PlaylistSongSchema objects representing the songs in the playlist.
    """
    playlist_songs = (
        db.query(PlaylistSong)
        .join(PlaylistSong.song)
        .filter(PlaylistSong.playlist_id == playlist_id)
        .all()
    )
    return [PlaylistSongSchema.from_orm(playlist_song) for playlist_song in playlist_songs]

def create_playlist_song(db: Session, playlist_song_data: PlaylistSongCreate) -> PlaylistSongSchema:
    """
    Creates a new association between a playlist and a song.

    Args:
        db (Session): The database session.
        playlist_song_data (PlaylistSongCreate): The data for the playlist-song association to create.

    Returns:
        PlaylistSongSchema: The newly created playlist-song association.
    """
    db_playlist_song = PlaylistSong(**playlist_song_data.dict())
    db.add(db_playlist_song)
    db.commit()
    db.refresh(db_playlist_song)
    return PlaylistSongSchema.from_orm(db_playlist_song)

def remove_song_from_playlist(db: Session, playlist_id: UUID, song_id: UUID) -> bool:
    """
    Removes a song from a playlist.

    Args:
        db (Session): The database session.
        playlist_id (UUID): The ID of the playlist.
        song_id (UUID): The ID of the song to remove.

    Returns:
        bool: True if the song was successfully removed, False otherwise.
    """
    playlist_song = (
        db.query(PlaylistSong)
        .filter(
            PlaylistSong.playlist_id == playlist_id,
            PlaylistSong.song_id == song_id,
        )
        .first()
    )
    if playlist_song:
        db.delete(playlist_song)
        db.commit()
        return True
    return False