from typing import List, Optional
from sqlalchemy.orm import Session
from uuid import UUID
from models.toy import Toy
from schemas.toy import ToyCreate, ToySchema

def get_all_toys(db: Session) -> List[ToySchema]:
    """
    Retrieves all toys from the database.

    Args:
        db (Session): The database session.

    Returns:
        List[ToySchema]: A list of all toy objects.
    """
    toys = db.query(Toy).all()
    return [ToySchema.from_orm(toy) for toy in toys]

def create_toy(db: Session, toy_data: ToyCreate) -> ToySchema:
    """
    Creates a new toy in the database.

    Args:
        db (Session): The database session.
        toy_data (ToyCreate): The data for the toy to create.

    Returns:
        ToySchema: The newly created toy object.
    """
    db_toy = Toy(**toy_data.dict())
    db.add(db_toy)
    db.commit()
    db.refresh(db_toy)
    return ToySchema.from_orm(db_toy)

def get_toy_by_id(db: Session, toy_id: UUID) -> Optional[ToySchema]:
    """
    Retrieves a single toy by its ID.

    Args:
        db (Session): The database session.
        toy_id (UUID): The ID of the toy to retrieve.

    Returns:
        Optional[ToySchema]: The toy object if found, otherwise None.
    """
    db_toy = db.query(Toy).filter(Toy.id == toy_id).first()
    return ToySchema.from_orm(db_toy) if db_toy else None